@@ -1,9 +1,10 @@
 """ configuration """
-from __future__ import print_function, unicode_literals
 import os
 import re
 import warnings
 
+from ._version_cls import NonNormalizedVersion
+from ._version_cls import Version
 from .utils import trace
 
 DEFAULT_TAG_REGEX = r"^(?:[\w-]+-)?(?P<version>[vV]?\d+(?:\.\d+){0,2}[^\+]*)(?:\+.*)?$"
@@ -48,8 +49,14 @@ def _check_absolute_root(root, relative_to):
     return os.path.abspath(root)
 
 
-class Configuration(object):
-    """ Global configuration model """
+def _lazy_tomli_load(data):
+    from tomli import loads
+
+    return loads(data)
+
+
+class Configuration:
+    """Global configuration model"""
 
     def __init__(
         self,
@@ -66,6 +73,9 @@ class Configuration(object):
         parse=None,
         git_describe_command=None,
         dist_name=None,
+        version_cls=None,
+        normalize=True,
+        search_parent_directories=False,
     ):
         # TODO:
         self._relative_to = relative_to
@@ -83,6 +93,32 @@ class Configuration(object):
         self.tag_regex = tag_regex
         self.git_describe_command = git_describe_command
         self.dist_name = dist_name
+        self.search_parent_directories = search_parent_directories
+        self.parent = None
+
+        if not normalize:
+            # `normalize = False` means `version_cls = NonNormalizedVersion`
+            if version_cls is not None:
+                raise ValueError(
+                    "Providing a custom `version_cls` is not permitted when "
+                    "`normalize=False`"
+                )
+            self.version_cls = NonNormalizedVersion
+        else:
+            # Use `version_cls` if provided, default to packaging or pkg_resources
+            if version_cls is None:
+                version_cls = Version
+            elif isinstance(version_cls, str):
+                try:
+                    # Not sure this will work in old python
+                    import importlib
+
+                    pkg, cls_name = version_cls.rsplit(".", 1)
+                    version_cls_host = importlib.import_module(pkg)
+                    version_cls = getattr(version_cls_host, cls_name)
+                except:  # noqa
+                    raise ValueError(f"Unable to import version_cls='{version_cls}'")
+            self.version_cls = version_cls
 
     @property
     def fallback_root(self):
@@ -127,14 +163,50 @@ class Configuration(object):
         self._tag_regex = _check_tag_regex(value)
 
     @classmethod
-    def from_file(cls, name="pyproject.toml", dist_name=None):
+    def from_file(
+        cls,
+        name: str = "pyproject.toml",
+        dist_name=None,  # type: str | None
+        _load_toml=_lazy_tomli_load,
+    ):
         """
         Read Configuration from pyproject.toml (or similar).
         Raises exceptions when file is not found or toml is
         not installed or the file has invalid format or does
         not contain the [tool.setuptools_scm] section.
         """
-        with open(name) as strm:
-            defn = __import__("toml").load(strm)
-        section = defn.get("tool", {})["setuptools_scm"]
+
+        with open(name, encoding="UTF-8") as strm:
+            data = strm.read()
+        defn = _load_toml(data)
+        try:
+            section = defn.get("tool", {})["setuptools_scm"]
+        except LookupError as e:
+            raise LookupError(
+                f"{name} does not contain a tool.setuptools_scm section"
+            ) from e
+        if "dist_name" in section:
+            if dist_name is None:
+                dist_name = section.pop("dist_name")
+            else:
+                assert dist_name == section["dist_name"]
+                del section["dist_name"]
+        if dist_name is None:
+            if "project" in defn:
+                # minimal pep 621 support for figuring the pretend keys
+                dist_name = defn["project"].get("name")
+        if dist_name is None:
+            dist_name = _read_dist_name_from_setup_cfg()
+
         return cls(dist_name=dist_name, **section)
+
+
+def _read_dist_name_from_setup_cfg():
+
+    # minimal effort to read dist_name off setup.cfg metadata
+    import configparser
+
+    parser = configparser.ConfigParser()
+    parser.read(["setup.cfg"])
+    dist_name = parser.get("metadata", "name", fallback=None)
+    return dist_name
